@@ -1,7 +1,8 @@
-// Copyright 2015 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2015 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 var LibraryIDBStore = {
   // A simple IDB-backed storage mechanism. Suitable for saving and loading large files asynchronously. This does
@@ -54,14 +55,13 @@ var LibraryIDBStore = {
     });
   },
 
-#if EMTERPRETIFY_ASYNC
-  emscripten_idb_load__deps: ['$EmterpreterAsync'],
+#if WASM_BACKEND && ASYNCIFY
   emscripten_idb_load: function(db, id, pbuffer, pnum, perror) {
-    EmterpreterAsync.handle(function(resume) {
+    Asyncify.handleSleep(function(wakeUp) {
       IDBStore.getFile(UTF8ToString(db), UTF8ToString(id), function(error, byteArray) {
         if (error) {
           {{{ makeSetValueAsm('perror', 0, '1', 'i32') }}};
-          resume();
+          wakeUp();
           return;
         }
         var buffer = _malloc(byteArray.length); // must be freed by the caller!
@@ -69,56 +69,52 @@ var LibraryIDBStore = {
         {{{ makeSetValueAsm('pbuffer', 0, 'buffer', 'i32') }}};
         {{{ makeSetValueAsm('pnum',    0, 'byteArray.length', 'i32') }}};
         {{{ makeSetValueAsm('perror',  0, '0', 'i32') }}};
-        resume();
+        wakeUp();
       });
     });
   },
-  emscripten_idb_store__deps: ['$EmterpreterAsync'],
   emscripten_idb_store: function(db, id, ptr, num, perror) {
-    EmterpreterAsync.handle(function(resume) {
+    Asyncify.handleSleep(function(wakeUp) {
       IDBStore.setFile(UTF8ToString(db), UTF8ToString(id), new Uint8Array(HEAPU8.subarray(ptr, ptr+num)), function(error) {
         {{{ makeSetValueAsm('perror', 0, '!!error', 'i32') }}};
-        resume();
+        wakeUp();
       });
     });
   },
-  emscripten_idb_delete__deps: ['$EmterpreterAsync'],
   emscripten_idb_delete: function(db, id, perror) {
-    EmterpreterAsync.handle(function(resume) {
+    Asyncify.handleSleep(function(wakeUp) {
       IDBStore.deleteFile(UTF8ToString(db), UTF8ToString(id), function(error) {
         {{{ makeSetValueAsm('perror', 0, '!!error', 'i32') }}};
-        resume();
+        wakeUp();
       });
     });
   },
-  emscripten_idb_exists__deps: ['$EmterpreterAsync'],
   emscripten_idb_exists: function(db, id, pexists, perror) {
-    EmterpreterAsync.handle(function(resume) {
+    Asyncify.handleSleep(function(wakeUp) {
       IDBStore.existsFile(UTF8ToString(db), UTF8ToString(id), function(error, exists) {
         {{{ makeSetValueAsm('pexists', 0, '!!exists', 'i32') }}};
         {{{ makeSetValueAsm('perror',  0, '!!error', 'i32') }}};
-        resume();
+        wakeUp();
       });
     });
   },
   // extra worker methods - proxied
-  emscripten_idb_load_blob__deps: ['$EmterpreterAsync'],
   emscripten_idb_load_blob: function(db, id, pblob, perror) {
-    EmterpreterAsync.handle(function(resume) {
+    Asyncify.handleSleep(function(wakeUp) {
       assert(!IDBStore.pending);
       IDBStore.pending = function(msg) {
         IDBStore.pending = null;
         var blob = msg.blob;
         if (!blob) {
           {{{ makeSetValueAsm('perror', 0, '1', 'i32') }}};
-          resume();
+          wakeUp();
           return;
         }
         assert(blob instanceof Blob);
         var blobId = IDBStore.blobs.length;
         IDBStore.blobs.push(blob);
         {{{ makeSetValueAsm('pblob', 0, 'blobId', 'i32') }}};
-        resume();
+        wakeUp();
       };
       postMessage({
         target: 'IDBStore',
@@ -127,31 +123,14 @@ var LibraryIDBStore = {
         id: UTF8ToString(id)
       });
     });
-    /*
-    EmterpreterAsync.handle(function(resume) {
-      IDBStore.getFile(UTF8ToString(db), UTF8ToString(id), function(error, blob) {
-        if (error) {
-          {{{ makeSetValueAsm('perror', 0, '1', 'i32') }}};
-          resume();
-          return;
-        }
-        assert(blob instanceof Blob);
-        var blobId = IDBStore.blobs.length;
-        IDBStore.blobs.push(blob);
-        {{{ makeSetValueAsm('pblob', 0, 'blobId', 'i32') }}};
-        resume();
-      });
-    });
-    */
   },
-  emscripten_idb_store_blob__deps: ['$EmterpreterAsync'],
   emscripten_idb_store_blob: function(db, id, ptr, num, perror) {
-    EmterpreterAsync.handle(function(resume) {
+    Asyncify.handleSleep(function(wakeUp) {
       assert(!IDBStore.pending);
       IDBStore.pending = function(msg) {
         IDBStore.pending = null;
         {{{ makeSetValueAsm('perror', 0, '!!msg.error', 'i32') }}};
-        resume();
+        wakeUp();
       };
       postMessage({
         target: 'IDBStore',
@@ -161,16 +140,7 @@ var LibraryIDBStore = {
         blob: new Blob([new Uint8Array(HEAPU8.subarray(ptr, ptr+num))])
       });
     });
-    /*
-    EmterpreterAsync.handle(function(resume) {
-      IDBStore.setFile(UTF8ToString(db), UTF8ToString(id), new Blob([new Uint8Array(HEAPU8.subarray(ptr, ptr+num))]), function(error) {
-        {{{ makeSetValueAsm('perror', 0, '!!error', 'i32') }}};
-        resume();
-      });
-    });
-    */
   },
-  emscripten_idb_read_from_blob__deps: ['$EmterpreterAsync'],
   emscripten_idb_read_from_blob: function(blobId, start, num, buffer) {
     var blob = IDBStore.blobs[blobId];
     if (!blob) return 1;
@@ -179,7 +149,6 @@ var LibraryIDBStore = {
     HEAPU8.set(new Uint8Array(byteArray), buffer);
     return 0;
   },
-  emscripten_idb_free_blob__deps: ['$EmterpreterAsync'],
   emscripten_idb_free_blob: function(blobId) {
     assert(IDBStore.blobs[blobId]);
     IDBStore.blobs[blobId] = null;
@@ -197,7 +166,7 @@ var LibraryIDBStore = {
   emscripten_idb_exists: function() {
     throw 'Please compile your program with async support in order to use synchronous operations like emscripten_idb_exists, etc.';
   },
-#endif
+#endif // WASM_BACKEND && ASYNCIFY
 };
 
 autoAddDeps(LibraryIDBStore, '$IDBStore');
