@@ -72,7 +72,9 @@ else:
   # closure compiler removes |var Module|, we need to find the closured name
   # seek a pattern like (e.ENVIRONMENT), which is in the shell.js if-cascade for the ENVIRONMENT override
   import re
-  m = re.search(r'(\w+)\s*=\s*"__EMSCRIPTEN_PRIVATE_MODULE_EXPORT_NAME_SUBSTITUTION__"', everything)
+  m = re.search(r'(\w+)\s*=\s*{"__EMSCRIPTEN_PRIVATE_MODULE_EXPORT_NAME_SUBSTITUTION__":1}', everything)
+  if not m:
+    m = re.search(r'(\w+)\s*=\s*{__EMSCRIPTEN_PRIVATE_MODULE_EXPORT_NAME_SUBSTITUTION__:1}', everything)
   if not m:
     m = re.search(r'(\w+)=typeof Module !== \'undefined\' \? Module : {}', everything)
   if not m:
